%% ------  Lezione3, 17 ottobre 2016 ------ 
% Standardizzazione dei dati

clear all; % cancella il contenuto di memoria
close all; % cancella tutte le figure presenti nella sessione
clc; % pulisce lo schermo

% Primo esempio: a cosa serve
% Costruisco un insieme di N punti 2D
% Ogni COLONNA di data e' un punto nello spazio euclideo, con le sue
% coordinate x e y. In altre parole, x e y sono le FEATURE che
% caratterizzano il punto.
N = 100;
data(1,:) = normrnd(0,1,[1 N]); % Valori sulle X dei punti
data(2,:) = 10*normrnd(0,1,[1 N]); % Valori sulle Y dei punti: la scala e' molto maggiore

% Comando scatter: visualizzo i miei dati
figure; subplot(1,2,1);
scatter(data(1,:),data(2,:)); axis equal % axis equal per rendersi conto della scala
xlabel('Feature 1'); ylabel('Feature 2');
title('Dati originali');

% Come cambiano i dati dopo una z-score standardization?
% Cerco di evitare i cicli for
% 1: calcolo la media
mu = mean(data,2); % Controllare l'help del comando mean

% 2: centro i dati
datac = data - repmat(mu, [1 N]); % Controllare l'help del comando repmat

% Soluzione alternativa 1:
% tmp = ones(1,N);
% datac = data - mu*tmp;

% Soluzione 2 (sconsigliata) con il ciclo for:
% for i = 1:N
%     datac(:,i) = data(:,i) - mu;
% end

% 3: calcolo la deviazione standard
sigma = std(data,[],2); % Controllare l'help del comando std

% 4: divido per la deviazione standard
datas = datac ./ repmat(sigma, [1 N]);

% Plot del risultato
subplot(1,2,2);
scatter(datas(1,:),datas(2,:)); axis equal
xlabel('Feature 1'); ylabel('Feature 2');
title('Dati standardizzati (z-score)');

%% ESERCIZIO
% Caricare in memoria la matrice "iris.mat", corrispondente al
% problema di classificazione di iris presentato nelle slide.
close all % Chiudo le figure, se ce ne sono
load('iris.mat')

% Estrarre la sottomatrice contenente tutti gli
% oggetti e solo le prime due feature, ed effettuarne uno scatterplot.
% Cosi' facendo, abbiamo creato un nuovo dataset con solo due feature.

data_sub = data(1:2,:);
scatter(data_sub(1,:), data_sub(2,:), [], labels)
title('Dataset originale (prime due feature)')

% Su questo dataset cosi' creato, provare a standardizzare i dati in modo tale che,
% ogni feature sia compresa in un intervallo fra 0 e 1.
% (è la standardizzazione domain (2) nelle slide)
% N.B. ogni feature va standardizzata separatamente!

m = min(data_sub, [], 2); % il minimo deve essere sulle features=righe di data_sub
M = max(data_sub, [], 2);
data_sub_std = (data_sub - repmat(m, [1 150])) ./ repmat(M-m, [1 150]);

% Effettuare uno scatter plot dei dati standardizzati 
figure,
scatter(data_sub_std(1,:), data_sub_std(2,:), [], labels);
title('Dataset standardizzato - range dei dati fra 0 e 1')
% Ora la visualizzazione mostra gli assi x e y sono compresi fra 0 e 1.

% Funzioni utili:
% - max
% - min
% Attenzione che per estrarre max e min sulle colonne la sintassi è
% min(data,[],2)






